/// Demonstration of dynamic page creation using the Smartware Web Server
///
/// There are a few samples here of the dynamically generated web pages.
/// Read the header above each one for more details.
///
#include "mbed.h"

#include "SW_HTTPServer.h"
#include "DynamicPages.h"
#include "Utility.h"


BusOut leds(LED4,LED3,LED2,LED1);       // dynamic page "/dyn2" lets you control these


/// SimplyDynamicPage1
///
/// This web page is generated dynamically as a kind of "bare minimum".
/// It doesn't do much.
///
/// You can see in main how this page was registered.
///
bool SuperSimpleDynamicPage(HTTPServer *svr, HTTPServer::CallBackType type, const char * path, const HTTPServer::namevalue *params, int paramcount)
{
    bool ret = false;
    char contentlen[30];
    #ifdef INCLUDE_GETREMOTE
    char remoteIP[36];
    #endif
    char buf[250];

    switch (type) {
        case HTTPServer::SEND_PAGE:
            // This sample drops it all into a local buffer, computes the length,
            // and passes that along as well. This can help the other end with efficiency.
            strcpy(buf, "<html><head><title>Dynamic Page</title></head>\r\n");
            strcat(buf, "<body>\r\n");
            strcat(buf, "<h1>Smart WiFly Web Server</h1>\r\n");
            strcat(buf, "This page was generated dynamically.<br/>\r\n");
            #ifdef INCLUDE_GETREMOTE
            svr->GetRemoteAddr(remoteIP, sizeof(remoteIP));
            strcat(buf, "Client IP Address: ");
            strcat(buf, remoteIP);
            strcat(buf, "<br/>\r\n");
            #endif
            strcat(buf, "<a href='/'>back to main</a></body></html>\r\n");
            sprintf(contentlen, "Content-Length: %d\r\n", strlen(buf));
            // Now the actual header response
            svr->header(200, "OK", "Content-Type: text/html\r\n", contentlen);
            // and data are sent
            svr->send(buf);
            ret = true;
            break;
        case HTTPServer::CONTENT_LENGTH_REQUEST:
            ret = true;
            break;
        case HTTPServer::DATA_TRANSFER:
            ret = true;
            break;
        default:
            ret = false;
            break;
    }
    return ret;
}


/// SimplyDynamicPage
///
/// This web page is generated dynamically and fills in part of the information from
/// data extracted from the URL as a query string. It also shows some additional
/// information - memory stats, server stats and so forth.
///
/// You can see in main how this page was registered.
///
bool SimpleDynamicPage(HTTPServer *svr, HTTPServer::CallBackType type, const char * path, const HTTPServer::namevalue *params, int paramcount)
{
    char buf[100];
    bool ret = false;
    HTTPServer::SW_PerformanceData perfData;

    switch (type) {
        case HTTPServer::SEND_PAGE:
            if (strcmp("true", svr->GetParameter("Reset")) == 0)
                svr->ResetPerformanceData();
            // Send the header
            svr->header(200, "OK", "Content-Type: text/html\r\n");

            // Send top of the page
            svr->send(
                "<html><head><title>Dynamic Page</title></head>\r\n"
                "<body>\r\n"
                "<h1>Smart WiFly Web Server</h1>\r\n"
                "This page was generated dynamically. Create your own name=value pairs on the URL "
                "which uses the GET method.<br/>\r\n"

            // Show Passed-in parameters
                "<h2>Query Parameters:</h2><blockquote>\r\n");
            sprintf(buf, "%d parameters passed to {%s}:<br/>\r\n", paramcount, path);
            svr->send(buf);
            // show each of the parameters passed on the URL
            for (int i=0; i<paramcount; i++) {
                sprintf(buf, "%d: %s = %s<br/>\r\n", i, params[i].name, params[i].value);
                svr->send(buf);
            }
            svr->send("</blockquote>\r\n");

            // Show Memory Stats
            svr->send(
                "<h2>System Statistics:</h2><blockquote>\r\n"
                "<table border='1'><tr><td>Parameter</td><td>Description</td></tr>\r\n");
            sprintf(buf,"<tr><td align='right'>%d</td><td>Free memory</td></tr>\r\n", Free());
            svr->send(buf);
            sprintf(buf,"<tr><td align='right'>%d</td><td>Max Header size</td></tr>\r\n", svr->GetMaxHeaderSize());
            svr->send(buf);
            sprintf(buf,"<tr><td align='right'>%3.2f</td><td>Wifly SW version</td></tr>\r\n", svr->GetWifly()->getWiflyVersion());
            svr->send(buf);
            sprintf(buf,"<tr><td align='right'>%s</td><td>Wifly Version Information</td></tr>\r\n", svr->GetWifly()->getWiflyVersionString());
            svr->send(buf);
            svr->send("</table>\r\n");
            svr->send("</blockquote>\r\n");

            // Show Web Server Performance metrics
            svr->GetPerformanceData(&perfData);
            svr->send("<h2>Web Server Performance Metrics</h2><blockquote>\r\n");
            svr->send("<table border='1'><tr><td>avg time (uS)</td><td>samples</td><td>max time (uS)</td><td>description</td></tr>\r\n");

            sprintf(buf, "<tr><td align='right'>%d</td><td align='right'>%d</td><td align='right'>%d</td><td>%s</td></tr>\r\n",
                    (unsigned long)(perfData.ConnectionAccepted.TotalTime_us / perfData.ConnectionAccepted.Samples),
                    perfData.ConnectionAccepted.Samples,
                    (unsigned long)(perfData.ConnectionAccepted.MaxTime_us),
                    "Connection Accepted");
            svr->send(buf);

            sprintf(buf, "<tr><td align='right'>%d</td><td align='right'>%d</td><td align='right'>%d</td><td>%s</td></tr>\r\n",
                    (unsigned long)(perfData.HeaderParsed.TotalTime_us / perfData.HeaderParsed.Samples),
                    perfData.HeaderParsed.Samples,
                    (unsigned long)(perfData.HeaderParsed.MaxTime_us),
                    "Header Parsed");
            svr->send(buf);
            
            sprintf(buf, "<tr><td align='right'>%d</td><td align='right'>%d</td><td align='right'>%d</td><td>%s</td></tr>\r\n",
                    (unsigned long)(perfData.ResponseSent.TotalTime_us / perfData.ResponseSent.Samples),
                    perfData.ResponseSent.Samples,
                    (unsigned long)(perfData.ResponseSent.MaxTime_us),
                    "Response Sent");
            svr->send(buf);
            
            sprintf(buf, "<tr><td align='right'>%d</td><td align='right'>%d</td><td align='right'>%d</td><td>%s</td></tr>\r\n",
                    (unsigned long)(perfData.ConnectionClosed.TotalTime_us / perfData.ConnectionClosed.Samples),
                    perfData.ConnectionClosed.Samples,
                    (unsigned long)(perfData.ConnectionClosed.MaxTime_us),
                    "Connection Closed");
            svr->send(buf);

            svr->send(
                "<tr><td colspan='3'>&nbsp;</td><td><a href='?Reset=false'>Reload</a>"
                " <a href='?Reset=true'>Reset Metrics</a></td></tr>\r\n"
                "</table>\r\n"
                "</blockquote>\r\n");

            // Send bottom of the page
            svr->send("<br/><a href='/'>back to main</a></body></html>\r\n");
            ret = true;
            break;
        case HTTPServer::CONTENT_LENGTH_REQUEST:
            ret = true;
            break;
        case HTTPServer::DATA_TRANSFER:
            ret = true;
            break;
        default:
            ret = false;
            break;
    }
    return ret;
}

/// SimplyDynamicForm
///
/// This web page is generated dynamically and fills in part of the information from
/// data extracted from the URL. It also generates a web form, and allows you to
/// update the data in the form using the POST method. This one also updates the
/// leds on the mbed module based on the form submission, so demonstrates a very
/// simple interaction with the hardware.
///
/// You can see in main how this page was registered.
///
bool SimpleDynamicForm(HTTPServer *svr, HTTPServer::CallBackType type, const char * path, const HTTPServer::namevalue *params, int paramcount)
{
    char buf[100];
    bool ret = false;

    switch (type) {
        case HTTPServer::SEND_PAGE:
            // set the LEDs based on a passed in parameter.
            leds = atoi(svr->GetParameter("leds"));
            // send the header
            svr->header(200, "OK", svr->GetSupportedType(".htm"));  //svr->header(200, "OK", "Content-Type: text/html\r\n");
            // send some data
            svr->send("<html><head><title>Dynamic Page</title></head>\r\n");
            svr->send("<body>\r\n");
            svr->send("<h1>Smart WiFly Web Server</h1>\r\n");
            svr->send("This form was generated dynamically. You can add name=value pairs on the URL "
                      "which will show up in the form, but the form is submitted using POST method.<br/>\r\n");
            svr->send("Hint: leds=7 turns on 3 of the 4 blue leds on the mbed<br/>\r\n");
            sprintf(buf, "%d parameters passed to {%s}:<br/>\r\n", paramcount, path);
            svr->send(buf);
            // Create a user form for which they can post changes
            sprintf(buf, "<form method='post' action='%s'>\r\n", path);
            //sprintf(buf, "<form method='post' enctype='multipart/form-data' action='%s'>\r\n", path); // not supported
            svr->send(buf);
            // show the parameters in a nice format
            svr->send("<table>\r\n");
            for (int i=0; i<paramcount; i++) {
                if (strcmp(params[i].name, "InFile") == 0)
                    continue;
                sprintf(buf, "<tr><td>%d</td><td>%s</td><td><input type='text' name='%s' value='%s'></td></tr>\r\n", i, params[i].name, params[i].name, params[i].value);
                svr->send(buf);
            }
            //svr->send("<tr><td>&nbsp;</td><td>File</td><td><input type='file' name='InFile' size='40'></td></tr>\r\n");
            svr->send("<tr><td>&nbsp;</td><td colspan='2'><input type='submit' value='submit'><input type='reset' value='clear'></td></tr>\r\n");
            svr->send("</table>\r\n");
            svr->send("</form>\r\n");
            // see how we're doing with free memory
            svr->send("<br/><a href='/'>Back to main</a></body></html>\r\n");
            ret = true;
            break;
        case HTTPServer::CONTENT_LENGTH_REQUEST:
            ret = true;
            break;
        case HTTPServer::DATA_TRANSFER:
            ret = true;
            break;
        default:
            ret = false;
            break;
    }
    return ret;
}

